@@ -23,6 +23,12 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: acctproc.c,v $
+** Revision 1.23  2007/03/20 13:01:51  gerlof
+** Introduction of variable supportflags.
+**
+** Revision 1.22  2007/02/13 09:14:49  gerlof
+** New boolean introduced to indicate if accounting is active.
+**
 ** Revision 1.21  2006/02/07 06:46:15  gerlof
 ** Removed swap-counter.
 **
@@ -91,7 +97,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: acctproc.c,v 1.21 2006/02/07 06:46:15 gerlof Exp $";
+static const char rcsid[] = "$Id: acctproc.c,v 1.23 2007/03/20 13:01:51 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -117,7 +123,7 @@ static	char 	acctatop;	/* boolean: atop's own accounting busy ?  */
 static	off_t	acctsize;	/* previous size of account file	  */
 static	int	acctrecsz;	/* size of account record		  */
 static	int	acctversion;	/* version of account record layout	  */
-int   		acctfd   = -1;	/* fd of account file       		  */
+static	int   	acctfd = -1;	/* fd of account file       		  */
 
 static count_t 	acctexp (comp_t  ct);
 static count_t	acctexp2(comp2_t ct);
@@ -202,7 +208,7 @@ acctswon(void)
 					** open active account file
 					*/
 					if ( (acctfd = open(pacctadm[i].name,
-							       O_RDONLY) ) < 0)
+							    O_RDONLY) ) == -1)
 					{
 						perror("open account-file");
 						fprintf(stderr,
@@ -221,6 +227,7 @@ acctswon(void)
 						return 0;
 					}
 
+					supportflags |= ACCTACTIVE;
 					return 1;
 				}
 			}
@@ -364,6 +371,7 @@ acctswon(void)
 
 	acctvers(acctfd);
 
+	supportflags |= ACCTACTIVE;
 	return 1;
 }
 
@@ -386,19 +394,19 @@ acctvers(int fd)
 	switch (tmprec.ac_version)
 	{
 	   case 3:
-	   	acctrecsz   = sizeof(struct acct_v3);
-		acctversion = 3;
+	   	acctrecsz     = sizeof(struct acct_v3);
+		acctversion   = 3;
 		break;
 
 	   case 6:
-	   	acctrecsz   = sizeof(struct acct_atop);
-		acctversion = 6;
-		kernpatch  |= PATCHACCT;  
+	   	acctrecsz     = sizeof(struct acct_atop);
+		acctversion   = 6;
+		supportflags |= PATCHACCT;  
 		break;
 
 	   default:
-	   	acctrecsz   = sizeof(struct acct);
-		acctversion = 2;
+	   	acctrecsz     = sizeof(struct acct);
+		acctversion   = 2;
 	}
 
 	/*
@@ -467,6 +475,8 @@ acctswoff(void)
 
 		(void) semop(semid, &semrelse, 1);
 	}
+
+	supportflags &= ~ACCTACTIVE;
 }
 
 /*
