@@ -115,6 +115,25 @@
 ** at runtime. 
 **
 ** $Log: atop.c,v $
+** Revision 1.29  2007/03/20 13:01:36  gerlof
+** Introduction of variable supportflags.
+**
+** Revision 1.28  2007/03/20 12:13:00  gerlof
+** Be sure that all pstat struct's are initialized with binary zeroes.
+**
+** Revision 1.27  2007/02/19 11:55:04  gerlof
+** Bug-fix: flag -S was not recognized any more.
+**
+** Revision 1.26  2007/02/13 10:34:20  gerlof
+** Support parseable output with flag -P
+**
+** Revision 1.25  2007/01/26 12:10:40  gerlof
+** Add configuration-value 'swoutcritsec'.
+**
+** Revision 1.24  2007/01/18 10:29:22  gerlof
+** Improved syntax-checking for ~/.atoprc file.
+** Support for network-interface busy-percentage.
+**
 ** Revision 1.23  2006/02/07 08:27:04  gerlof
 ** Cosmetic changes.
 **
@@ -188,7 +207,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: atop.c,v 1.23 2006/02/07 08:27:04 gerlof Exp $";
+static const char rcsid[] = "$Id: atop.c,v 1.29 2007/03/20 13:01:36 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -209,47 +228,49 @@ static const char rcsid[] = "$Id: atop.c,v 1.23 2006/02/07 08:27:04 gerlof Exp $
 
 #include "atop.h"
 #include "acctproc.h"
+#include "ifprop.h"
 #include "photoproc.h"
 #include "photosyst.h"
 #include "showgeneric.h"
+#include "parseable.h"
 
-#define	allflags	"ab:cde:fghijklmnopqrstuvwxyz1ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-#define	PROCCHUNK	100	/* portion-size for process-table expansion */
-#define	MAXFL		64      /* maximum number of command-line flags     */
+#define	allflags  "ab:cde:fghijklmnopqrstuvwxyz1ABCDEFGHIJKLMNOP:QRSTUVWXYZ"
+#define	PROCCHUNK	50	/* process-entries for future expansion  */
+#define	MAXFL		64      /* maximum number of command-line flags  */
 
 /*
-** installed kernel patches
+** declaration of global variables
 */
-char		kernpatch;
-
-/*
-** timing info
-*/
-time_t 		curtime;	/* might be used external by other functions */
-time_t 		pretime;	/* might be used external by other functions */
-
-static char	awaittrigger;	/* boolean: awaiting trigger for next cycle  */
-
-/*
-** argument values and global variables
-*/
-unsigned long	interval = 10;
-unsigned int 	nsamples = 0xffffffff;
-long		sampcnt;
-int		deviatonly  = 1;
-
-char		flaglist[MAXFL];
 struct utsname	utsname;
-
+time_t 		pretime;	/* timing info				*/
+time_t 		curtime;	/* timing info				*/
+unsigned long	interval = 10;
+char		screen;
+char		acctactive;	/* accounting active (boolean)		*/
 char		rawname[RAWNAMESZ];
 char		rawreadflag;
-char		midnightflag;
-unsigned int	begintime, endtime;
-
+char		flaglist[MAXFL];
+char		deviatonly  = 1;
+unsigned short	hertz;
+unsigned int	pagesize;
 int 		osrel;
 int 		osvers;
 int 		ossub;
 
+int		supportflags;	/* supported features             	*/
+
+struct visualize vis = {generic_samp, generic_error, 
+		        generic_end,  generic_usage};
+
+/*
+** argument values
+*/
+static char		awaittrigger;	/* boolean: awaiting trigger */
+static unsigned int 	nsamples = 0xffffffff;
+static long		sampcnt;
+static char		midnightflag;
+static unsigned int	begintime, endtime;
+
 /*
 ** interpretation of defaults-file $HOME/.atop
 */
@@ -260,30 +281,34 @@ void do_procname(char *);
 void do_maxcpu(char *);
 void do_maxdisk(char *);
 void do_maxintf(char *);
+void do_cpucritperc(char *);
+void do_memcritperc(char *);
+void do_swpcritperc(char *);
+void do_dskcritperc(char *);
+void do_netcritperc(char *);
+void do_swoutcritsec(char *);
+void do_almostcrit(char *);
 
-struct {
+static struct {
 	char	*tag;
 	void	(*func)(char *);
 } manrc[] = {
-	{	"flags",	do_flags	},
-	{	"interval",	do_interval	},
-	{	"username",	do_username	},
-	{	"procname",	do_procname	},
-	{	"maxlinecpu",	do_maxcpu	},
-	{	"maxlinedisk",	do_maxdisk	},
-	{	"maxlineintf",	do_maxintf	},
+	{	"flags",		do_flags		},
+	{	"interval",		do_interval		},
+	{	"username",		do_username		},
+	{	"procname",		do_procname		},
+	{	"maxlinecpu",		do_maxcpu		},
+	{	"maxlinedisk",		do_maxdisk		},
+	{	"maxlineintf",		do_maxintf		},
+	{	"cpucritperc",		do_cpucritperc		},
+	{	"memcritperc",		do_memcritperc		},
+	{	"swpcritperc",		do_swpcritperc		},
+	{	"dskcritperc",		do_dskcritperc		},
+	{	"netcritperc",		do_netcritperc		},
+	{	"swoutcritsec",		do_swoutcritsec		},
+	{	"almostcrit",		do_almostcrit		},
 };
 
-/*
-** next struct contains pointers to four visualization-functions:
-**   1.	show all info about one sample
-**   2.	show error-message
-**   3.	handle end-of-program
-**   4.	show usage-info
-*/
-struct visualize vis = {generic_samp, generic_error, 
-		        generic_end,  generic_usage};
-
 /*
 ** internal prototypes
 */
@@ -293,7 +318,7 @@ int
 main(int argc, char *argv[])
 {
 	register int	i;
-	int		c;
+	int		c, nr, line=0;
 	char		*p;
 	struct rlimit	rlim;
 
@@ -320,13 +345,47 @@ main(int argc, char *argv[])
 
 			while ( fgets(linebuf, sizeof linebuf, fp) )
 			{
-				if ( sscanf(linebuf, "%15s %15s",
-							tagname, tagvalue) < 2)
+				line++;
+
+				nr = sscanf(linebuf, "%15s %15s",
+							tagname, tagvalue);
+
+				switch (nr)
+				{
+				   case 0:
 					continue;
 
-				if (tagname[0] == '#' || tagvalue[0] == '#')
-					continue;
+				   case 1:
+					if (tagname[0] == '#')
+						continue;
 
+					fprintf(stderr,
+						"~/.atoprc: syntax error line "
+						"%d (no value specified)\n",
+						line);
+
+					cleanstop(1);
+					break;		/* not reached */
+
+				   default:
+					if (tagname[0] == '#')
+						continue;
+					
+					if (tagvalue[0] != '#')
+						break;
+
+					fprintf(stderr,
+						"~/.atoprc: syntax error line "
+						"%d (no value specified)\n",
+						line);
+
+					cleanstop(1);
+				}
+
+				/*
+				** tag name and tag value found
+				** try to recognize tag name
+				*/
 				for (i=0; i < sizeof manrc/sizeof manrc[0]; i++)
 				{
 					if ( strcmp(tagname, manrc[i].tag) ==0)
@@ -335,6 +394,19 @@ main(int argc, char *argv[])
 						break;
 					}
 				}
+
+				/*
+				** tag name not recognized
+				*/
+				if (i == sizeof manrc/sizeof manrc[0])
+				{
+					fprintf(stderr,
+						"~/.atoprc: syntax error line "
+						"%d (tag name %s not valid)\n",
+						line, tagname);
+
+					cleanstop(1);
+				}
 			}
 
 			fclose(fp);
@@ -396,6 +468,13 @@ main(int argc, char *argv[])
 					prusage(argv[0]);
 				break;
 
+                           case 'P':		/* parseable output?          */
+				if ( !parsedef(optarg) )
+					prusage(argv[0]);
+
+				vis.show_samp = parseout;
+				break;
+
 			   default:		/* gather other flags */
 				flaglist[i++] = c;
 			}
@@ -435,6 +514,12 @@ main(int argc, char *argv[])
 
 	sscanf(utsname.release, "%d.%d.%d", &osrel, &osvers, &ossub);
 
+	/*
+	** determine the clock rate and memory page size for this machine
+	*/
+	hertz		= sysconf(_SC_CLK_TCK);
+	pagesize	= sysconf(_SC_PAGESIZE);
+
 	/*
 	** check if raw data from a file must be viewed
 	*/
@@ -446,8 +531,7 @@ main(int argc, char *argv[])
 	}
 
 	/*
-	** determine time since when current statistics have
-	** been maintained
+	** determine start-time for gathering current statistics
 	*/
 	curtime = getboot();
 
@@ -482,7 +566,12 @@ main(int argc, char *argv[])
 	acctswon();
 
 	/*
-	** since process-accounting is switched on now, there is no
+	** determine properties (like speed) of all interfaces
+	*/
+	initifprop();
+
+	/*
+	** since priviliged activities are finished now, there is no
 	** need to keep running under root-priviliges, so switch
 	** effective user-id to real user-id
 	*/
@@ -534,7 +623,10 @@ engine(void)
 	presstat = calloc(1, sizeof(struct sstat));
 	devsstat = calloc(1, sizeof(struct sstat));
 
-	if (cursstat == NULL || presstat == NULL || devsstat == NULL)
+	curplen = countprocs() + PROCCHUNK;
+	curpact = calloc(curplen, sizeof(struct pstat));
+
+	if (!cursstat || !presstat || !devsstat || !curpact)
 	{
 		fprintf(stderr, "unexpected calloc-failure...\n");
 		cleanstop(1);
@@ -630,9 +722,11 @@ engine(void)
 		**  --> atop malloc's a minimal amount of space which is
 		**      only extended when needed
 		*/
+		memset(curpact, 0, curplen * sizeof(struct pstat));
+
 		while ( (npresent = photoproc(curpact, curplen)) == curplen)
 		{
-			curplen += PROCCHUNK;
+			curplen = countprocs() + PROCCHUNK;
 
 			curpact = realloc(curpact,
 			                   curplen * sizeof(struct pstat));
