@@ -36,17 +36,6 @@
 
 typedef	long long	count_t;
 
-/*
-** bit-values for installed kernel-patches
-*/
-extern char		kernpatch;
-
-#define	PATCHSTAT	0x01
-#define	PATCHACCT	0x02
-
-/*
-** structure containing the start-addresses of functions for visualization
-*/
 struct pstat;
 struct sstat;
 
@@ -58,8 +47,40 @@ struct visualize {
 	void	(*show_usage) (void);
 };
 
+/*
+** external values
+*/
+extern struct utsname   utsname;
+extern time_t   	pretime;
+extern time_t   	curtime;
+extern unsigned long    interval;
+extern char      	screen;
+extern char		deviatonly;
+extern char		rawname[];
+extern char		rawreadflag;
+extern char		flaglist[];
 extern struct visualize vis;
 
+extern int      	osrel;
+extern int		osvers;
+extern int      	ossub;
+
+extern unsigned short	hertz;
+extern unsigned int	pagesize;
+
+extern int		supportflags;
+
+/*
+** bit-values for supportflags
+*/
+#define	ACCTACTIVE	0x00000001
+#define	PATCHSTAT	0x00000002
+#define	PATCHACCT	0x00000003
+#define	IOSTAT		0x00000004
+
+/*
+** structure containing the start-addresses of functions for visualization
+*/
 char		generic_samp(time_t, int, struct sstat *, struct pstat *,
 		             int, int, int, int, char);
 void		generic_error(const char *, ...);
@@ -89,7 +110,7 @@ int		cpucompar (const void *, const void *);
 int		diskcompar(const void *, const void *);
 int		intfcompar(const void *, const void *);
 
-count_t 	subcount(count_t, count_t);
+count_t		subcount(count_t, count_t);
 void  		rawread(unsigned int, unsigned int);
 char		rawwrite(time_t, int, struct sstat *, struct pstat *,
 		                       int, int, int, int, char);
@@ -97,7 +118,7 @@ char		rawwrite(time_t, int, struct sstat *, struct pstat *,
 int 		numeric(char *);
 void		getalarm(int);
 time_t		getboot(void);
-unsigned int	getpagesz(void);
-char 		*getversion(void);
+char 		*getstrvers(void);
+unsigned short 	getnumvers(void);
 void		cleanstop(int);
 void		prusage(char *);
