@@ -30,6 +30,24 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: deviate.c,v $
+** Revision 1.31  2007/03/20 13:02:03  gerlof
+** Introduction of variable supportflags.
+**
+** Revision 1.30  2007/03/20 11:18:57  gerlof
+** Add counter for cancelled writes.
+**
+** Revision 1.29  2007/02/13 09:21:04  gerlof
+** Removed external declarations.
+**
+** Revision 1.28  2007/01/22 08:28:18  gerlof
+** Support steal-time from /proc/stat.
+**
+** Revision 1.27  2007/01/18 10:43:18  gerlof
+** Support for network-interface busy-percentage (speed and duplex).
+**
+** Revision 1.26  2006/11/13 13:47:26  gerlof
+** Implement load-average counters, context-switches and interrupts.
+**
 ** Revision 1.25  2006/02/07 06:45:33  gerlof
 ** Removed swap-counter.
 **
@@ -107,7 +125,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: deviate.c,v 1.25 2006/02/07 06:45:33 gerlof Exp $";
+static const char rcsid[] = "$Id: deviate.c,v 1.31 2007/03/20 13:02:03 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -123,11 +141,10 @@ static const char rcsid[] = "$Id: deviate.c,v 1.25 2006/02/07 06:45:33 gerlof Ex
 #include <string.h>
 
 #include "atop.h"
+#include "ifprop.h"
 #include "photoproc.h"
 #include "photosyst.h"
 
-extern time_t	pretime;	/* previous sample time 	 */
-
 #define	MAX32BITVAL	0x100000000LL
 
 /*
@@ -238,6 +255,8 @@ deviatproc(struct pstat *aproc, int npresent,
 			subcount(curstat->dsk.wio, prestat.dsk.wio);
 		devstat->dsk.wsz    =
 			subcount(curstat->dsk.wsz, prestat.dsk.wsz);
+		devstat->dsk.cwsz   =
+			subcount(curstat->dsk.cwsz, prestat.dsk.cwsz);
 
 		devstat->mem.shtext = curstat->mem.shtext;
 		devstat->mem.vmem   = curstat->mem.vmem;
@@ -339,7 +358,7 @@ deviatproc(struct pstat *aproc, int npresent,
 		devstat->cpu.curcpu   = 0;
 		devstat->cpu.sleepavg = 0;
 
-		if (kernpatch & PATCHACCT)
+		if (supportflags & PATCHACCT)
 		{
 			devstat->net.tcpsnd = curstat->net.tcpsnd -
 			                      prestat.net.tcpsnd;
@@ -369,6 +388,8 @@ deviatproc(struct pstat *aproc, int npresent,
 			                      prestat.dsk.wio;
 			devstat->dsk.wsz    = curstat->dsk.wsz    -
 			                      prestat.dsk.wsz;
+			devstat->dsk.cwsz   = curstat->dsk.cwsz    -
+			                      prestat.dsk.cwsz;
 			devstat->mem.vmem   = 0;
 			devstat->mem.rmem   = 0;
 			devstat->mem.vgrow  = -curstat->mem.vmem;
@@ -392,6 +413,7 @@ deviatproc(struct pstat *aproc, int npresent,
 			devstat->mem.rgrow    = 0;
 			devstat->dsk.wio      = 0;
 			devstat->dsk.wsz      = 0;
+			devstat->dsk.cwsz     = 0;
 			devstat->dsk.rsz      = 0;
 			devstat->dsk.rio      = curstat->dsk.rio  -
 						prestat.dsk.rio   -
@@ -423,12 +445,14 @@ deviatproc(struct pstat *aproc, int npresent,
 			devstat->dsk.wio   = 0;
 		if (devstat->dsk.wsz    < 0)
 			devstat->dsk.wsz   = 0;
+		if (devstat->dsk.cwsz    < 0)
+			devstat->dsk.cwsz   = 0;
 		if (devstat->mem.minflt < 0)
 			devstat->mem.minflt = 0;
 		if (devstat->mem.majflt < 0)
 			devstat->mem.majflt = 0;
 
-		if (kernpatch & PATCHACCT)
+		if (supportflags & PATCHACCT)
 		{
 			if (devstat->net.tcpsnd < 0)
 				 devstat->net.tcpsnd = 0;
@@ -478,6 +502,8 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 	dev->cpu.all.Itime = subcount(cur->cpu.all.Itime, pre->cpu.all.Itime);
 	dev->cpu.all.Stime = subcount(cur->cpu.all.Stime, pre->cpu.all.Stime);
 
+	dev->cpu.all.steal = subcount(cur->cpu.all.steal, pre->cpu.all.steal);
+
 	if (dev->cpu.nrcpu == 1)
 	{
 		dev->cpu.cpu[0] = dev->cpu.all;
@@ -501,9 +527,16 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 						         pre->cpu.cpu[i].Itime);
 			dev->cpu.cpu[i].Stime = subcount(cur->cpu.cpu[i].Stime,
 						         pre->cpu.cpu[i].Stime);
+
+			dev->cpu.cpu[i].steal = subcount(cur->cpu.cpu[i].steal,
+						         pre->cpu.cpu[i].steal);
 		}
 	}
 
+	dev->cpu.lavg1		= cur->cpu.lavg1;
+	dev->cpu.lavg5		= cur->cpu.lavg5;
+	dev->cpu.lavg15		= cur->cpu.lavg15;
+
 	dev->mem.physmem	= cur->mem.physmem;
 	dev->mem.freemem	= cur->mem.freemem;
 	dev->mem.buffermem	= cur->mem.buffermem;
@@ -529,10 +562,24 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 	dev->net.udpin 		= subcount(cur->net.udpin,   pre->net.udpin);
 	dev->net.udpout		= subcount(cur->net.udpout,  pre->net.udpout);
 
+	/*
+	** calculate deviations for interfaces
+	*/
 	if (pre->intf.intf[0].name[0] == '\0')	/* first sample? */
 	{
+		struct ifprop	ifprop;
+
 		for (i=0; cur->intf.intf[i].name[0]; i++)
+		{
 			strcpy(pre->intf.intf[i].name, cur->intf.intf[i].name);
+
+			strcpy(ifprop.name, cur->intf.intf[i].name);
+
+			getifprop(&ifprop);
+
+			pre->intf.intf[i].speed         = ifprop.speed;
+			pre->intf.intf[i].duplex        = ifprop.fullduplex;
+ 		}
 	}
 	
 	for (i=0; cur->intf.intf[i].name[0]; i++)
@@ -542,8 +589,36 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 		** in that case, skip further handling for this sample
 		*/
 		if (strcmp(cur->intf.intf[i].name, pre->intf.intf[i].name) != 0)
-			break;
+		{
+			int		j;
+			struct ifprop	ifprop;
 
+			/*
+			** take care that interface properties are
+			** corrected for future samples
+			*/
+                        seteuid(0);		/* get root privileges      */
+
+		        initifprop();		/* refresh interface info   */
+
+                        seteuid( getuid() );	/* release root privileges  */
+
+			for (j=0; cur->intf.intf[j].name[0]; j++)
+			{
+				strcpy(ifprop.name, cur->intf.intf[j].name);
+
+				getifprop(&ifprop);
+
+				cur->intf.intf[j].speed  = ifprop.speed;
+				cur->intf.intf[j].duplex = ifprop.fullduplex;
+			}
+
+			break;
+		}
+
+		/*
+		** calculate interface deviations for this sample
+		*/
 		strcpy(dev->intf.intf[i].name, cur->intf.intf[i].name);
 
 		dev->intf.intf[i].rbyte = subcount(cur->intf.intf[i].rbyte,
@@ -554,11 +629,23 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 		                                  pre->intf.intf[i].sbyte);
 		dev->intf.intf[i].spack = subcount(cur->intf.intf[i].spack,
 		                                  pre->intf.intf[i].spack);
+
+		dev->intf.intf[i].speed 	= pre->intf.intf[i].speed;
+		dev->intf.intf[i].duplex	= pre->intf.intf[i].duplex;
+	
+		/*
+		** save interface properties for next interval
+		*/
+		cur->intf.intf[i].speed 	= pre->intf.intf[i].speed;
+		cur->intf.intf[i].duplex	= pre->intf.intf[i].duplex;
 	}
 
 	dev->intf.intf[i].name[0] = '\0';
 	dev->intf.nrintf = i;
 
+	/*
+	** calculate deviations for disks
+	*/
 	for (i=0; cur->xdsk.xdsk[i].name[0]; i++)
 	{
 		strcpy(dev->xdsk.xdsk[i].name, cur->xdsk.xdsk[i].name);
