@@ -31,6 +31,18 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photoproc.c,v $
+** Revision 1.28  2007/03/27 10:53:59  gerlof
+** Bug-solution: only allow IOSTAT when patches are not installed.
+**
+** Revision 1.27  2007/03/21 14:21:37  gerlof
+** Handle io counters maintained from 2.6.20
+**
+** Revision 1.26  2007/02/13 10:32:34  gerlof
+** Removal of external declarations.
+**
+** Revision 1.25  2007/01/15 09:00:14  gerlof
+** Add new function to count actual number of processes.
+**
 ** Revision 1.24  2006/02/07 06:47:35  gerlof
 ** Removed swap-counter.
 **
@@ -108,7 +120,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photoproc.c,v 1.24 2006/02/07 06:47:35 gerlof Exp $";
+static const char rcsid[] = "$Id: photoproc.c,v 1.28 2007/03/27 10:53:59 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -168,11 +180,24 @@ photoproc(struct pstat *proclist, int maxproc)
 			(void) fgets(line, sizeof line, fp);
 
 			if ( fgets(line, sizeof line, fp) != NULL)
-				kernpatch |= PATCHSTAT;
+				supportflags |= PATCHSTAT;
 
 			fclose(fp);
 		}
 
+		/*
+		** check if this kernel offers io-statistics per process
+		*/
+		if ( !(supportflags & PATCHSTAT) )
+		{
+			if ( (fp = fopen("/proc/1/io", "r")) )
+			{
+				supportflags |= IOSTAT;
+
+				fclose(fp);
+			}
+		}
+
 		firstcall = 0;
 	}
 
@@ -212,8 +237,7 @@ photoproc(struct pstat *proclist, int maxproc)
 		pval++;
 
 		/*
-		** store the full command line;
-		** the command-line may contain:
+		** store the full command line; the command-line may contain:
 		**    - null-bytes as a separator between the arguments
 		**    - newlines (e.g. arguments for awk or sed)
 		**    - tabs (e.g. arguments for awk or sed)
@@ -243,55 +267,6 @@ photoproc(struct pstat *proclist, int maxproc)
 			}
 		}
 
-#if	0
-		/*
-		** Linux >= 2.6
-		** For multithreaded processes, the subdirectory 'task'
-		** contains "pid's" again for the threads related to
-		** this process.
-		**
-		** This piece of code is not valid any more due to modified
-		** accounting by the kernel.
-		*/ 
-		if ( nthreads > 1 && chdir("task") == 0 )
-		{
-			DIR		*dirtaskp;
-			struct dirent	*enttaskp;
-
-			dirtaskp = opendir(".");
-
-			while ( (enttaskp = readdir(dirtaskp))
-							&& pval < maxproc)
-			{
-				/*
-				** skip non-numerical names
-				*/
-				if (!isdigit(enttaskp->d_name[0]))
-					continue;
-
-				/*
-				** change to the underlying process' subdir
-				*/
-				if ( chdir(enttaskp->d_name) != 0 )
-					continue;
-
-				/*
-				** call fillproc to open the required files
-				** and gather the process' info
-				*/
-				curproc	= proclist+pval;
-
-				if ( fillproc(curproc) )
-					pval++;
-
-				chdir("..");
-			}
-
-			closedir(dirtaskp);
-			chdir("..");
-		}
-#endif
-
 		chdir("..");
 	}
 
@@ -315,7 +290,7 @@ fillproc(struct pstat *curproc)
 	count_t		utime, stime, starttime;
 	count_t		minflt, majflt, size, rss, nswap,
 			startcode, endcode,
-			dskrio=0, dskwio=0, dskrsz=0, dskwsz=0,
+			dskrio=0, dskwio=0, dskrsz=0, dskwsz=0, dskcwsz=0,
 			tcpsnd=0, tcprcv=0, tcpssz=0, tcprsz=0,
 			udpsnd=0, udprcv=0, udpssz=0, udprsz=0,
 			rawsnd=0, rawrcv=0;
@@ -407,21 +382,59 @@ fillproc(struct pstat *curproc)
 
 	fclose(fp);
 
+	/*
+	** open file "io" (>= 2.6.20) and obtain required info
+	*/
+#define	IO_READ		"read_bytes:"
+#define	IO_WRITE	"write_bytes:"
+#define	IO_CWRITE	"cancelled_write_bytes:"
+
+	if ((supportflags & IOSTAT) && (fp = fopen("io", "r")) )
+	{
+		while (fgets(line, sizeof line, fp))
+		{
+			if (memcmp(line, IO_READ, sizeof IO_READ -1) == 0)
+			{
+				sscanf(line, "%*s %llu", &dskrsz);
+				dskrsz /= 512;		/* in sectors     */
+				dskrio  = dskrsz;	/* enable sorting */
+				continue;
+			}
+
+			if (memcmp(line, IO_WRITE, sizeof IO_WRITE -1) == 0)
+			{
+				sscanf(line, "%*s %llu", &dskwsz);
+				dskwsz /= 512;		/* in sectors     */
+				dskwio  = dskwsz;	/* enable sorting */
+				continue;
+			}
+
+			if (memcmp(line, IO_CWRITE, sizeof IO_CWRITE -1) == 0)
+			{
+				sscanf(line, "%*s %llu", &dskcwsz);
+				dskcwsz /= 512;		/* in sectors     */
+				continue;
+			}
+		}
+
+		fclose(fp);
+	}
+
 	/*
 	** store required info in process-structure
 	*/
-	curproc->gen.pid   = pid;
-	curproc->gen.ruid  = ruid;
-	curproc->gen.rgid  = rgid;
-	curproc->gen.tgid  = tgid;
-	curproc->gen.nthr  = nthreads;
-	curproc->gen.state = state;
+	curproc->gen.pid      = pid;
+	curproc->gen.ruid     = ruid;
+	curproc->gen.rgid     = rgid;
+	curproc->gen.tgid     = tgid;
+	curproc->gen.nthr     = nthreads;
+	curproc->gen.state    = state;
 
 	strncpy(curproc->gen.name, command, PNAMLEN);
 	curproc->gen.name[PNAMLEN] = 0;
 
 	curproc->gen.excode   = 0;
-	curproc->gen.btime    = starttime/HZ+bootepoch;
+	curproc->gen.btime    = starttime/hertz+bootepoch;
 	curproc->cpu.utime    = utime;
 	curproc->cpu.stime    = stime;
 	curproc->cpu.nice     = nice;
@@ -431,28 +444,60 @@ fillproc(struct pstat *curproc)
 	curproc->cpu.curcpu   = curcpu;
 	curproc->cpu.sleepavg = sleepavg;
 
-	curproc->mem.minflt = minflt;
-	curproc->mem.majflt = majflt;
-	curproc->mem.vmem   = size / 1024;
-	curproc->mem.rmem   = rss  * (getpagesz()/1024);
-	curproc->mem.vgrow  = 0;	/* calculated later */
-	curproc->mem.rgrow  = 0;	/* calculated later */
-	curproc->mem.shtext = (endcode-startcode)/1024;
+	curproc->mem.minflt   = minflt;
+	curproc->mem.majflt   = majflt;
+	curproc->mem.vmem     = size / 1024;
+	curproc->mem.rmem     = rss  * (pagesize/1024);
+	curproc->mem.vgrow    = 0;	/* calculated later */
+	curproc->mem.rgrow    = 0;	/* calculated later */
+	curproc->mem.shtext   = (endcode-startcode)/1024;
 
-	curproc->dsk.rio    = dskrio;
-	curproc->dsk.rsz    = dskrsz;
-	curproc->dsk.wio    = dskwio;
-	curproc->dsk.wsz    = dskwsz;
-	curproc->net.tcpsnd = tcpsnd;
-	curproc->net.tcpssz = tcpssz;
-	curproc->net.tcprcv = tcprcv;
-	curproc->net.tcprsz = tcprsz;
-	curproc->net.udpsnd = udpsnd;
-	curproc->net.udpssz = udpssz;
-	curproc->net.udprcv = udprcv;
-	curproc->net.udprsz = udprsz;
-	curproc->net.rawsnd = rawsnd;
-	curproc->net.rawrcv = rawrcv;
+	curproc->dsk.rio      = dskrio;
+	curproc->dsk.rsz      = dskrsz;
+	curproc->dsk.wio      = dskwio;
+	curproc->dsk.wsz      = dskwsz;
+	curproc->dsk.cwsz     = dskcwsz;
+	curproc->net.tcpsnd   = tcpsnd;
+	curproc->net.tcpssz   = tcpssz;
+	curproc->net.tcprcv   = tcprcv;
+	curproc->net.tcprsz   = tcprsz;
+	curproc->net.udpsnd   = udpsnd;
+	curproc->net.udpssz   = udpssz;
+	curproc->net.udprcv   = udprcv;
+	curproc->net.udprsz   = udprsz;
+	curproc->net.rawsnd   = rawsnd;
+	curproc->net.rawrcv   = rawrcv;
 
 	return nthreads;
 }
+
+/*
+** count number of processes currently running
+*/
+unsigned int
+countprocs(void)
+{
+	unsigned int	nr=0;
+	DIR		*dirp;
+	struct dirent	*entp;
+	char		origdir[1024];
+
+	getcwd(origdir, sizeof origdir);
+	chdir("/proc");
+	dirp = opendir(".");
+
+	while ( (entp = readdir(dirp)) )
+	{
+		/*
+		** count subdirectory-names starting with a digit
+		*/
+		if (isdigit(entp->d_name[0]))
+			nr++;
+	}
+
+	closedir(dirp);
+
+	chdir(origdir);
+
+	return nr;
+}
