@@ -43,8 +43,7 @@ struct pstat {
 		int	excode;		/* process exit status		*/
 		time_t 	btime;		/* process start time (epoch)	*/
 		char	cmdline[CMDLEN+1];/* command-line string       	*/
-		int	future1;	/* reserved for future use	*/
-		int	future2;	/* reserved for future use	*/
+		int	ifuture[4];	/* reserved for future use	*/
 	} gen;
 
 	/* CPU STATISTICS						*/
@@ -57,8 +56,8 @@ struct pstat {
 		int	policy;		/* scheduling policy            */
 		int	curcpu;		/* current processor            */
 		int	sleepavg;       /* sleep average percentage     */
-		count_t	future1;	/* reserved for future use	*/
-		count_t	future2;	/* reserved for future use	*/
+		int	ifuture[4];	/* reserved for future use	*/
+		count_t	cfuture[4];	/* reserved for future use	*/
 	} cpu;
 
 	/* DISK STATISTICS						*/
@@ -67,13 +66,13 @@ struct pstat {
 		count_t	rsz;		/* cumulative # sectors read	*/
 		count_t	wio;		/* number of write requests 	*/
 		count_t	wsz;		/* cumulative # sectors written	*/
-		count_t	future1;	/* reserved for future use	*/
-		count_t	future2;	/* reserved for future use	*/
+		count_t	cwsz;		/* cumulative # written sectors */
+					/* being cancelled              */
+		count_t	cfuture[4];	/* reserved for future use	*/
 	} dsk;
 
 	/* MEMORY STATISTICS						*/
 	struct mem {
-		count_t	future1;	/* reserved for future use	*/
 		count_t	minflt;		/* number of page-reclaims 	*/
 		count_t	majflt;		/* number of page-faults 	*/
 		count_t	shtext;		/* text     memory (Kb)         */
@@ -81,8 +80,7 @@ struct pstat {
 		count_t	rmem;		/* resident memory (Kb)		*/
 		count_t vgrow;		/* virtual  growth (Kb)    	*/
 		count_t rgrow;		/* resident growth (Kb)     	*/
-		count_t	future2;	/* reserved for future use	*/
-		count_t	future3;	/* reserved for future use	*/
+		count_t	cfuture[4];	/* reserved for future use	*/
 	} mem;
 
 	/* NETWORK STATISTICS						*/
@@ -97,8 +95,7 @@ struct pstat {
 		count_t udprsz;		/* cumulative size packets sent	*/
 		count_t	rawsnd;		/* number of raw packets sent	*/
 		count_t	rawrcv;		/* number of raw packets recved	*/
-		count_t	future1;	/* reserved for future use	*/
-		count_t	future2;	/* reserved for future use	*/
+		count_t	cfuture[4];	/* reserved for future use	*/
 	} net;
 };
 
@@ -113,17 +110,18 @@ struct pinfo {
 /*
 ** prototypes of process-database functions
 */
-int	 pdb_getproc(int, struct pinfo **);
-void	 pdb_addproc(int, struct pinfo *);
-int	 pdb_delproc(int);
-int	 pdb_newproc(struct pinfo **);
-int	 pdb_makeresidue(void);
-int	 pdb_cleanresidue(void);
-int	 pdb_srchresidue(struct pstat *, struct pinfo **);
+int		pdb_getproc(int, struct pinfo **);
+void		pdb_addproc(int, struct pinfo *);
+int		pdb_delproc(int);
+int		pdb_newproc(struct pinfo **);
+int		pdb_makeresidue(void);
+int		pdb_cleanresidue(void);
+int		pdb_srchresidue(struct pstat *, struct pinfo **);
 
 /*
 ** prototypes for raw process-statistics functions
 */
-int	deviatproc(struct pstat *, int, struct pstat *, int, int,
+int		deviatproc(struct pstat *, int, struct pstat *, int, int,
 					struct pstat *, int *);
-int	photoproc(struct pstat *, int);
+int		photoproc(struct pstat *, int);
+unsigned int	countprocs(void);
