@@ -30,6 +30,21 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photosyst.c,v $
+** Revision 1.26  2007/02/13 10:32:28  gerlof
+** Removal of external declarations.
+**
+** Revision 1.25  2007/02/13 09:29:57  gerlof
+** Removal of external declarations.
+**
+** Revision 1.24  2007/01/22 14:57:12  gerlof
+** Support of special disks used by virtual machines.
+**
+** Revision 1.23  2007/01/22 08:28:50  gerlof
+** Support steal-time from /proc/stat.
+**
+** Revision 1.22  2006/11/13 13:48:20  gerlof
+** Implement load-average counters, context-switches and interrupts.
+**
 ** Revision 1.21  2006/01/30 09:14:16  gerlof
 ** Extend memory counters (a.o. page scans).
 **
@@ -96,7 +111,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photosyst.c,v 1.21 2006/01/30 09:14:16 gerlof Exp $";
+static const char rcsid[] = "$Id: photosyst.c,v 1.26 2007/02/13 10:32:28 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -116,18 +131,15 @@ static const char rcsid[] = "$Id: photosyst.c,v 1.21 2006/01/30 09:14:16 gerlof
 
 static int	isrealdisk(char *, char *, int);
 
-extern int	osrel;
-extern int	osvers;
-
 void
 photosyst(struct sstat *si)
 {
 	register int	i, nr, nlines;
 	count_t		cnts[MAXCNT];
+	float		lavg1, lavg5, lavg15;
 	FILE 		*fp;
 	char		linebuf[1024], nam[64], origdir[1024];
 	static char	part_stats = 1; /* per-partition statistics ? */
-	unsigned int	pgsz = getpagesz();
 
 	memset(si, 0, sizeof(struct sstat));
 
@@ -183,6 +195,9 @@ photosyst(struct sstat *si)
 					si->cpu.all.wtime	= cnts[4];
 					si->cpu.all.Itime	= cnts[5];
 					si->cpu.all.Stime	= cnts[6];
+
+					if (nr > 8)	/* steal support */
+						si->cpu.all.steal = cnts[7];
 				}
 				continue;
 			}
@@ -202,6 +217,9 @@ photosyst(struct sstat *si)
 					si->cpu.cpu[i].wtime	= cnts[4];
 					si->cpu.cpu[i].Itime	= cnts[5];
 					si->cpu.cpu[i].Stime	= cnts[6];
+
+					if (nr > 8)	/* steal support */
+						si->cpu.cpu[i].steal = cnts[7];
 				}
 
 				si->cpu.nrcpu++;
@@ -234,6 +252,26 @@ photosyst(struct sstat *si)
 			si->cpu.nrcpu = 1;
 	}
 
+	/*
+	** gather loadaverage values from the file /proc/loadavg and
+	** store them in binary form
+	*/
+	if ( (fp = fopen("loadavg", "r")) != NULL)
+	{
+		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			if ( sscanf(linebuf, "%f %f %f",
+				&lavg1, &lavg5, &lavg15) == 3)
+			{
+				si->cpu.lavg1	= lavg1;
+				si->cpu.lavg5	= lavg5;
+				si->cpu.lavg15	= lavg15;
+			}
+		}
+
+		fclose(fp);
+	}
+
 	/*
 	** gather virtual memory statistics from the file /proc/vmstat and
 	** store them in binary form (>= kernel 2.6)
@@ -322,15 +360,15 @@ photosyst(struct sstat *si)
 
 			if ( strcmp("Mem:", nam) == EQ)
 			{
-				si->mem.physmem	 	= cnts[0] / pgsz; 
-				si->mem.freemem		= cnts[2] / pgsz;
-				si->mem.buffermem	= cnts[4] / pgsz;
+				si->mem.physmem	 	= cnts[0] / pagesize; 
+				si->mem.freemem		= cnts[2] / pagesize;
+				si->mem.buffermem	= cnts[4] / pagesize;
 				nrfields -= 3;
 			}
 			else	if ( strcmp("Swap:", nam) == EQ)
 				{
-					si->mem.totswap  = cnts[0] / pgsz;
-					si->mem.freeswap = cnts[2] / pgsz;
+					si->mem.totswap  = cnts[0] / pagesize;
+					si->mem.freeswap = cnts[2] / pagesize;
 					nrfields -= 2;
 				}
 			else	if (strcmp("Cached:", nam) == EQ)
@@ -338,7 +376,7 @@ photosyst(struct sstat *si)
 					if (si->mem.cachemem  == (count_t)-1)
 					{
 						si->mem.cachemem  =
-							cnts[0]*1024/pgsz;
+							cnts[0]*1024/pagesize;
 						nrfields--;
 					}
 				}
@@ -347,7 +385,7 @@ photosyst(struct sstat *si)
 					if (si->mem.physmem  == (count_t)-1)
 					{
 						si->mem.physmem  =
-							cnts[0]*1024/pgsz;
+							cnts[0]*1024/pagesize;
 						nrfields--;
 					}
 				}
@@ -356,7 +394,7 @@ photosyst(struct sstat *si)
 					if (si->mem.freemem  == (count_t)-1)
 					{
 						si->mem.freemem  =
-							cnts[0]*1024/pgsz;
+							cnts[0]*1024/pagesize;
 						nrfields--;
 					}
 				}
@@ -365,7 +403,7 @@ photosyst(struct sstat *si)
 					if (si->mem.buffermem  == (count_t)-1)
 					{
 						si->mem.buffermem  =
-							cnts[0]*1024/pgsz;
+							cnts[0]*1024/pagesize;
 						nrfields--;
 					}
 				}
@@ -374,7 +412,7 @@ photosyst(struct sstat *si)
 					if (si->mem.totswap  == (count_t)-1)
 					{
 						si->mem.totswap  =
-							cnts[0]*1024/pgsz;
+							cnts[0]*1024/pagesize;
 						nrfields--;
 					}
 				}
@@ -383,23 +421,25 @@ photosyst(struct sstat *si)
 					if (si->mem.freeswap  == (count_t)-1)
 					{
 						si->mem.freeswap  =
-							cnts[0]*1024/pgsz;
+							cnts[0]*1024/pagesize;
 						nrfields--;
 					}
 				}
 			else	if (strcmp("Slab:", nam) == EQ)
 				{
-					si->mem.slabmem = cnts[0]*1024/pgsz;
+					si->mem.slabmem = cnts[0]*1024/pagesize;
 					nrfields--;
 				}
 			else	if (strcmp("Committed_AS:", nam) == EQ)
 				{
-					si->mem.committed = cnts[0]*1024/pgsz;
+					si->mem.committed = cnts[0]*1024/
+								pagesize;
 					nrfields--;
 				}
 			else	if (strcmp("CommitLimit:", nam) == EQ)
 				{
-					si->mem.commitlim = cnts[0]*1024/pgsz;
+					si->mem.commitlim = cnts[0]*1024/
+								pagesize;
 					nrfields--;
 				}
 		}
@@ -685,6 +725,10 @@ static struct {
 	{	"^rd/c[0-9][0-9]*d[0-9][0-9]*$",	{0},	nullmodname, },
 	{	"^cciss/c[0-9][0-9]*d[0-9][0-9]*$",	{0},	nullmodname, },
 	{ 	"/host.*/bus.*/target.*/lun.*/disc",	{0},	abbrevname1, },
+
+	/* virtual disks */
+	{	"^xvd[a-z][a-z]*$",			{0},	nullmodname, },
+	{	"^dasd[a-z][a-z]*$",			{0},	nullmodname, },
 };
 
 static int
