@@ -3,8 +3,6 @@
 **
 ** The program 'atop' offers the possibility to view the activity of 
 ** the system on system-level as well as process-level.
-**
-** This source-file contains the 
 ** ==========================================================================
 ** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
 ** E-mail:      gerlof@ATComputing.nl
@@ -28,6 +26,24 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: rawlog.c,v $
+** Revision 1.20  2007/03/20 13:02:25  gerlof
+** Introduction of variable supportflags.
+**
+** Revision 1.19  2007/03/20 11:11:57  gerlof
+** Verify success of malloc's.
+**
+** Revision 1.18  2007/03/20 07:25:59  gerlof
+** Avoid loop when incompatible raw file is read.
+** Verify return code of compress/uncompress functions.
+**
+** Revision 1.17  2007/02/23 07:34:00  gerlof
+** Changed unsigned short's into unsigned int's for process-counters
+** in raw record.
+**
+** Revision 1.16  2007/02/13 10:33:27  gerlof
+** Removal of external declarations.
+** Store pagesize and hertz in raw logfile.
+**
 ** Revision 1.15  2006/01/30 09:12:34  gerlof
 ** Minor bug-fix.
 **
@@ -78,7 +94,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: rawlog.c,v 1.15 2006/01/30 09:12:34 gerlof Exp $";
+static const char rcsid[] = "$Id: rawlog.c,v 1.20 2007/03/20 13:02:25 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -102,17 +118,6 @@ static const char rcsid[] = "$Id: rawlog.c,v 1.15 2006/01/30 09:12:34 gerlof Exp
 
 #define	BASEPATH	"/var/log/atop/"  
 
-/*
-** argument values and global variables
-*/
-extern unsigned long	interval;
-extern struct utsname	utsname;
-extern char		rawname[];
-
-extern int		osrel;
-extern int		osvers;
-extern int		ossub;
-
 /*
 ** structure which describes the raw file contents
 **
@@ -132,38 +137,50 @@ extern int		ossub;
 
 struct rawheader {
 	unsigned int	magic;
-	unsigned short	sstatlen;
-	unsigned short	pstatlen;
-	unsigned short	rawreclen;
-	char		acctfd;
-	char		kernpatch;
-	int		osrel;
-	int		osvers;
-	int		ossub;
-	char		node[32];
-	char		reserved[16];
+
+	unsigned short	aversion;	/* creator atop version with MSB */
+	unsigned short	sstatlen;	/* length of struct sstat        */
+	unsigned short	pstatlen;	/* length of struct pstat        */
+	unsigned short	rawheadlen;	/* length of struct rawheader    */
+	unsigned short	rawreclen;	/* length of struct rawrecord    */
+	unsigned short	hertz;		/* clock interrupts per second   */
+	unsigned short	sfuture[5];	/* future use                    */
+
+	char		node[32];	/* name of this system           */
+	char		cfuture[8];	/* future use                    */
+
+	unsigned int	pagesize;	/* size of memory page (bytes)   */
+	int		supportflags;  	/* used features                 */
+	int		osrel;		/* OS release number             */
+	int		osvers;		/* OS version number             */
+	int		ossub;		/* OS version subnumber          */
+	int		ifuture[6];	/* future use                    */
 };
 
 struct rawrecord {
 	time_t		curtime;	/* current time (epoch)         */
-	unsigned int	interval;	/* interval (number of seconds) */
+
 	unsigned short	flags;		/* various flags                */
-	unsigned short	nlist;		/* number of processes in list  */
-	unsigned short	npresent;	/* total number of processes    */
-	unsigned short	nexit;		/* number of exited processes   */
-	unsigned short	nzombie;	/* number of zombie processes   */
-	unsigned short	scomplen;	/* length of compressed sstat   */
+	unsigned short	sfuture[3];	/* future use                   */
+
+	unsigned int	scomplen;	/* length of compressed sstat   */
 	unsigned int	pcomplen;	/* length of compressed pstat's */
-	unsigned char	reserved[4];	/* future extensions            */
+	unsigned int	interval;	/* interval (number of seconds) */
+	unsigned int	nlist;		/* number of processes in list  */
+	unsigned int	npresent;	/* total number of processes    */
+	unsigned int	nexit;		/* number of exited processes   */
+	unsigned int	nzombie;	/* number of zombie processes   */
+	unsigned int	ifuture[6];	/* future use                   */
 };
 
 #define	RRBOOT	0x0001	/* rawrecord.flags				*/
 
 static int	getrawrec  (int, struct rawrecord *, int);
-static int	getrawsstat(int, struct sstat *, int, int);
-static int	getrawpstat(int, struct pstat *, int, int, int);
+static int	getrawsstat(int, struct sstat *, int);
+static int	getrawpstat(int, struct pstat *, int, int);
 static int	rawwopen(void);
 static int	lookslikedatetome(char *);
+static void	testcompval(int, char *);
 
 /*
 ** write a raw record to file
@@ -177,6 +194,7 @@ rawwrite(time_t curtime, int numsecs,
 {
 	static int		rawfd = -1;
 	struct rawrecord	rr;
+	int			rv;
 
 	Byte			scompbuf[sizeof(struct sstat)], *pcompbuf;
 	unsigned long		scomplen = sizeof scompbuf;
@@ -192,17 +210,26 @@ rawwrite(time_t curtime, int numsecs,
 	/*
 	** compress system- and process-level statistics
 	*/
-	(void) compress(scompbuf, &scomplen,
+	rv = compress(scompbuf, &scomplen,
 				(Byte *)ss, (unsigned long)sizeof *ss);
 
-	pcompbuf = malloc(pcomplen);
+	testcompval(rv, "compress");
 
-	(void) compress(pcompbuf, &pcomplen,
-				(Byte *)ps, (unsigned long)pcomplen);
+	if ( (pcompbuf = malloc(pcomplen)) == NULL)
+	{
+		perror("atop malloc");
+		cleanstop(7);
+	}
+
+	rv = compress(pcompbuf, &pcomplen, (Byte *)ps, (unsigned long)pcomplen);
+
+	testcompval(rv, "compress");
 
 	/*
 	** fill record header and write to file
 	*/
+	memset(&rr, 0, sizeof rr);
+
 	rr.curtime	= curtime;
 	rr.interval	= numsecs;
 	rr.flags	= 0;
@@ -213,8 +240,6 @@ rawwrite(time_t curtime, int numsecs,
 	rr.scomplen	= scomplen;
 	rr.pcomplen	= pcomplen;
 
-	memset(rr.reserved, 0, sizeof rr.reserved);
-
 	if (bootflag)
 		rr.flags |= RRBOOT;
 
@@ -268,7 +293,6 @@ static int
 rawwopen()
 {
 	struct rawheader	rh;
-	extern int		acctfd;
 	int			fd;
 
 	/*
@@ -285,16 +309,36 @@ rawwopen()
 			cleanstop(7);
 		}
 
-		if ( rh.magic		!= MYMAGIC			||
-		     rh.sstatlen	!= sizeof(struct sstat)		||
-		     rh.pstatlen	!= sizeof(struct pstat)		||
-		     rh.rawreclen	!= sizeof(struct rawrecord)	||
-		     rh.acctfd		!= acctfd			||
-		     rh.kernpatch	!= kernpatch			  )
+		if (rh.magic != MYMAGIC)
 		{
 			fprintf(stderr,
-				"%s exists and has incompatible header\n",
+				"file %s exists but does not contain raw "
+				"atop output (wrong magic number)\n", rawname);
+
+			cleanstop(7);
+		}
+
+		if ( rh.sstatlen	!= sizeof(struct sstat)		||
+		     rh.pstatlen	!= sizeof(struct pstat)		||
+	    	     rh.rawheadlen	!= sizeof(struct rawheader)	||
+		     rh.rawreclen	!= sizeof(struct rawrecord)	||
+		     rh.supportflags	!= supportflags			  )
+		{
+			fprintf(stderr,
+				"%s exists but has incompatible header\n",
 				rawname);
+
+			if (rh.aversion & 0x8000)
+			{
+				fprintf(stderr,
+					"(created by atop version %d.%d - "
+					"current atop version %d.%d)\n",
+					(rh.aversion >> 8) & 0x7f,
+					 rh.aversion & 0xff,
+					 getnumvers() >> 8,
+					 getnumvers() & 0x7f);
+			}
+
 			cleanstop(7);
 		}
 
@@ -304,7 +348,7 @@ rawwopen()
 	}
 
 	/*
-	** file does not exist already (or can not be opened)
+	** file does not exist (or can not be opened)
 	*/
 	if ( (fd = creat(rawname, 0666)) == -1)
 	{
@@ -313,21 +357,24 @@ rawwopen()
 		cleanstop(7);
 	}
 
+	memset(&rh, 0, sizeof rh);
+
 	rh.magic	= MYMAGIC;
+	rh.aversion	= getnumvers() | 0x8000;
 	rh.sstatlen	= sizeof(struct sstat);
 	rh.pstatlen	= sizeof(struct pstat);
+	rh.rawheadlen	= sizeof(struct rawheader);
 	rh.rawreclen	= sizeof(struct rawrecord);
-	rh.acctfd	= acctfd;
-	rh.kernpatch	= kernpatch;
+	rh.supportflags	= supportflags;
 	rh.osrel	= osrel;
 	rh.osvers	= osvers;
 	rh.ossub	= ossub;
+	rh.hertz	= hertz;
+	rh.pagesize	= pagesize;
 
 	strncpy(rh.node, utsname.nodename, sizeof rh.node);
 	rh.node[sizeof rh.node -1] = '\0';
 
-	memset(rh.reserved, 0, sizeof rh.reserved);
-
 	if ( write(fd, &rh, sizeof rh) == -1)
 	{
 		fprintf(stderr, "%s - ", rawname);
@@ -347,7 +394,6 @@ void
 rawread(unsigned int begintime, unsigned int endtime)
 {
 	int			rawfd;
-	extern int		acctfd;
 	struct rawheader	rh;
 	struct rawrecord	rr;
 	struct sstat		devsstat;
@@ -443,7 +489,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 	}
 
 	/*
-	** read the raw header and verify the magic number and version
+	** read the raw header and verify the magic
 	*/
 	if ( read(rawfd, &rh, sizeof rh) < sizeof rh)
 	{
@@ -458,20 +504,55 @@ rawread(unsigned int begintime, unsigned int endtime)
 		cleanstop(7);
 	}
 
+	/*
+	** magic okay, but file-layout might have been modified
+	*/
+	if (rh.sstatlen   != sizeof(struct sstat)		||
+	    rh.pstatlen   != sizeof(struct pstat)		||
+	    rh.rawheadlen != sizeof(struct rawheader)		||
+	    rh.rawreclen  != sizeof(struct rawrecord)		  )
+	{
+		fprintf(stderr,
+			"raw file %s has incompatible format\n", rawname);
+
+		if (rh.aversion & 0x8000)
+		{
+			fprintf(stderr,
+				"(created by atop version %d.%d - "
+				"current atop version %d.%d)\n",
+				(rh.aversion >> 8) & 0x7f,
+				 rh.aversion & 0xff,
+				 getnumvers() >> 8,
+				 getnumvers() & 0x7f);
+		}
+
+		cleanstop(7);
+	}
+
 	strncpy(utsname.nodename, rh.node, sizeof utsname.nodename);
 	utsname.nodename[sizeof utsname.nodename -1] = '\0';
 
-	kernpatch = rh.kernpatch;
-	acctfd    = rh.acctfd;
-	osrel     = rh.osrel;
-	osvers    = rh.osvers;
-	ossub     = rh.ossub;
-	interval  = 0;
+	supportflags = rh.supportflags;
+	osrel        = rh.osrel;
+	osvers       = rh.osvers;
+	ossub        = rh.ossub;
+	interval     = 0;
+
+	if (rh.hertz)
+		hertz    = rh.hertz;
+
+	if (rh.pagesize)
+		pagesize = rh.pagesize;
 
 	/*
 	** allocate a list for backtracking of rawrecord-offsets
 	*/
-	offlist = malloc(sizeof(off_t) * OFFCHUNK);
+	if ( (offlist = malloc(sizeof(off_t) * OFFCHUNK)) == NULL)
+	{
+		perror("atop malloc");
+		cleanstop(7);
+	}
+
 	offsize = OFFCHUNK;
 
 	*offlist = lseek(rawfd, 0, SEEK_CUR);
@@ -495,8 +576,13 @@ rawread(unsigned int begintime, unsigned int endtime)
 
 			if ( ++offcur >= offsize )
 			{
-				offlist = realloc(offlist,
-					     (offsize+OFFCHUNK)*sizeof(off_t));
+				if ( (offlist = realloc(offlist,
+				     (offsize+OFFCHUNK)*sizeof(off_t))) ==NULL)
+				{
+					perror("atop realloc");
+					cleanstop(7);
+				}
+
 				offsize+= OFFCHUNK;
 			}
 	
@@ -516,18 +602,22 @@ rawread(unsigned int begintime, unsigned int endtime)
 			** allocate space, read compressed system-level
 			** statistics and decompress
 			*/
-			if ( !getrawsstat(rawfd, &devsstat,
-						rr.scomplen, rh.sstatlen) )
+			if ( !getrawsstat(rawfd, &devsstat, rr.scomplen) )
 				cleanstop(7);
 
 			/*
 			** allocate space, read compressed process-level
 			** statistics and decompress
 			*/
-			devpstat  = malloc(sizeof(struct pstat) * rr.nlist);
+			if ( (devpstat =
+			       malloc(sizeof(struct pstat) * rr.nlist)) ==NULL)
+			{
+				perror("atop malloc");
+				cleanstop(7);
+			}
 
 			if ( !getrawpstat(rawfd, devpstat,
-					rr.pcomplen, rh.pstatlen, rr.nlist) )
+					rr.pcomplen, rr.nlist) )
 				cleanstop(7);
 
 			/*
@@ -575,55 +665,33 @@ rawread(unsigned int begintime, unsigned int endtime)
 static int
 getrawrec(int rawfd, struct rawrecord *prr, int rrlen)
 {
-	int			n;
-
-	switch ( rrlen )
-	{
-		/*
-		** rawrecord layout identical to the one used
-		** by this atop-version
-		*/
-	   case sizeof(struct rawrecord):
-		n = read(rawfd, prr, rrlen);
-		break;
-
-	   default:
-		fprintf(stderr, "raw file contains unrecognized format "
-				"of record structure!\n");
-		n = 0;
-	}
-
-	return n;
+	return read(rawfd, prr, rrlen);
 }
 
 /*
 ** read the system-level statistics from the current offset
 */
 static int
-getrawsstat(int rawfd, struct sstat *sp, int complen, int sslen)
+getrawsstat(int rawfd, struct sstat *sp, int complen)
 {
-	Byte			*compbuf;
-	unsigned long		uncomplen;
+	Byte		*compbuf;
+	unsigned long	uncomplen = sizeof(struct sstat);
+	int		rv;
 
-	compbuf = malloc(complen);
+	if ( (compbuf = malloc(complen)) == NULL)
+	{
+		perror("atop malloc");
+		cleanstop(7);
+	}
 
 	if ( read(rawfd, compbuf, complen) < complen)
 		return 0;
 
-	switch (sslen)
-	{
-	   case sizeof(struct sstat):
-		uncomplen = sslen;
-		(void) uncompress((Byte *)sp, &uncomplen, compbuf, complen);
-		free(compbuf);
-		break;
+	rv = uncompress((Byte *)sp, &uncomplen, compbuf, complen);
 
-	   default:
-		fprintf(stderr, "raw file contains unrecognized format "
-				"of system statistics!\n");
-		free(compbuf);
-		return 0;
-	}
+	testcompval(rv, "uncompress");
+
+	free(compbuf);
 
 	return 1;
 }
@@ -632,29 +700,26 @@ getrawsstat(int rawfd, struct sstat *sp, int complen, int sslen)
 ** read the process-level statistics from the current offset
 */
 static int
-getrawpstat(int rawfd, struct pstat *pp, int complen, int pslen, int nlist)
+getrawpstat(int rawfd, struct pstat *pp, int complen, int nlist)
 {
-	Byte			*compbuf;
-	unsigned long		uncomplen = pslen * nlist;
+	Byte		*compbuf;
+	unsigned long	uncomplen = sizeof(struct pstat) * nlist;
+	int		rv;
 
-	compbuf   = malloc(complen);
+	if ( (compbuf = malloc(complen)) == NULL)
+	{
+		perror("atop malloc");
+		cleanstop(7);
+	}
 
 	if ( read(rawfd, compbuf, complen) < complen)
 		return 0;
 
-	switch (pslen)
-	{
-	   case sizeof(struct pstat):
-		(void) uncompress((Byte *)pp, &uncomplen, compbuf, complen);
-		free(compbuf);
-		break;
+	rv = uncompress((Byte *)pp, &uncomplen, compbuf, complen);
 
-	   default:
-		fprintf(stderr, "raw file contains unrecognized format "
-				"of process statistics!\n");
-		free(compbuf);
-		return 0;
-	}
+	testcompval(rv, "uncompress");
+
+	free(compbuf);
 
 	return 1;
 }
@@ -682,3 +747,38 @@ lookslikedatetome(char *p)
 
 	return 1;	/* yes, looks like a date to me */
 }
+
+static void
+testcompval(int rv, char *func)
+{
+	switch (rv)
+	{
+	   case Z_OK:
+	   case Z_STREAM_END:
+	   case Z_NEED_DICT:
+		break;
+
+	   case Z_MEM_ERROR:
+		fprintf(stderr,
+		        "atop - %s: failed due to lack of memory\n",
+			func);
+		cleanstop(7);
+
+	   case Z_BUF_ERROR:
+		fprintf(stderr, 
+		        "atop - %s: failed due to lack of room in out buffer\n",
+			func);
+		cleanstop(7);
+
+   	   case Z_DATA_ERROR:
+		fprintf(stderr,
+		        "atop - %s: failed due to corrupted/incomplete data\n",
+			func);
+		cleanstop(7);
+
+	   default:
+		fprintf(stderr,
+		        "atop - %s: unexpected error %d\n", func, rv);
+		cleanstop(7);
+	}
+}
