@@ -30,6 +30,10 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: various.c,v $
+** Revision 1.14  2007/02/13 10:32:47  gerlof
+** Removal of external declarations.
+** Removal of function getpagesz().
+**
 ** Revision 1.13  2006/02/07 08:27:21  gerlof
 ** Add possibility to show counters per second.
 ** Modify presentation of CPU-values.
@@ -72,7 +76,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: various.c,v 1.13 2006/02/07 08:27:21 gerlof Exp $";
+static const char rcsid[] = "$Id: various.c,v 1.14 2007/02/13 10:32:47 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -403,12 +407,10 @@ getboot(void)
 	if (!boottime)		/* do this only once */
 	{
 #ifdef	linux
-		long		hertz = sysconf(_SC_CLK_TCK);
 		time_t		getbootlinux(long);
 
 		boottime = getbootlinux(hertz);
 #else
-		long		hertz = sysconf(_SC_CLK_TCK);
 		struct tms	tms;
 		time_t 		boottime_again;
 
@@ -430,22 +432,6 @@ getboot(void)
 	return boottime;
 }
 
-/*
-** Function getpagesz() returns the size of a memory-page of this system
-** (as number of bytes).
-*/
-unsigned int
-getpagesz(void)
-{
-	static time_t   pagesize;
-
-	if (pagesize==0)                /* do this only once */
-		pagesize = sysconf(_SC_PAGESIZE);
-
-	return pagesize;
-}
-
-
 /*
 ** signal catcher for cleanup before exit
 */
