@@ -2,13 +2,13 @@
 #include <stdio.h>
 #include <string.h>
 
-static const char rcsid[] = "$Id: version.c,v 1.16 2006/04/03 08:39:37 root Exp root $";
+static const char rcsid[] = "$Id: version.c,v 1.20 2007/03/27 11:07:08 root Exp root $";
 
-static char atoprevision[] = "$Revision: 1.16 $";
-static char atopdate[]     = "$Date: 2006/04/03 08:39:37 $";
+static char atoprevision[] = "$Revision: 1.20 $";
+static char atopdate[]     = "$Date: 2007/03/27 11:07:08 $";
 
 char *
-getversion(void)
+getstrvers(void)
 {
 	static char vers[256];
 
@@ -22,3 +22,13 @@ getversion(void)
 
 	return vers;
 }
+
+unsigned short
+getnumvers(void)
+{
+	int	vers1, vers2;
+
+	sscanf(atoprevision, "%*s %u.%u", &vers1, &vers2);
+
+	return (unsigned short) ((vers1 << 8) + vers2);
+}
